#include "GroupFromAnnotation.h"
#include "Error.h"
#include "MathVector.h"
#include "IntArray.h"

String GroupFromAnnotation::groupFile = "";
String GroupFromAnnotation::vcfInput = "";
String GroupFromAnnotation::function = "";


GroupFromAnnotation::GroupFromAnnotation()
{}

GroupFromAnnotation::~GroupFromAnnotation()
{
//   if(SNPlist) delete [] SNPlist;
//   if(SNPNoAllele) delete [] SNPNoAllele;
}

void GroupFromAnnotation::GetGroupFromFile()
{
   //Fill in annoGroups.
   StringArray tmp;
   FILE * file = fopen(groupFile,"r");
   if(file==NULL)
   {
      error("ERROR! Cannot open group file %s.\n",groupFile.c_str());
   }
   String buffer;
   int line = 0;
   while (!feof(file))
   {
      buffer.ReadLine(file);
      tmp.Clear();
      tmp.AddTokens(buffer, SEPARATORS);
      if(tmp.Length()==0)
	 continue;
      annoGroups.Push(tmp[0]);
      chrom.Push(tmp[1]);
      line++;
   }
   fclose(file);

   //Fill in SNPlist.
   SNPlist = new StringArray [line];
   SNPNoAllele = new StringArray [line];
   FILE * samefile = fopen(groupFile,"r");
   line = 0;
   Vector pos;
   while (!feof(samefile))
   {
      buffer.ReadLine(samefile);
      tmp.Clear();
      pos.Clear();
      tmp.AddTokens(buffer, "\t ");
      SNPlist[line].Dimension(0);
      SNPNoAllele[line].Dimension(0);
      for(int i=1;i<tmp.Length();i++)
      {
	 SNPlist[line].Push(tmp[i]);
	 StringArray sub;
	 sub.Clear();
	 sub.AddTokens(tmp[i],":");
	 pos.Push(sub[1].AsInteger());
	 SNPNoAllele[line].Push(sub[0] + ":" + sub[1]);
      }
      //sort SNPlist[line] and SNPNoAllele[line]
      if(SNPlist[line].Length()>1)
      {
	 Vector sorted_pos,order;
	 sorted_pos.Copy(pos);
	 sorted_pos.Sort();
	 order.Dimension(pos.Length());
	 for(int i=0;i<sorted_pos.Length();i++)
	 {
	    for(int j=0;j<pos.Length();j++)
	    {
	       if(sorted_pos[i]==pos[j])
	       {
		  order[i]=j; 
		  break;
	       }
	    }
	 }

	 StringArray cp_SNPlist,cp_SNPNoAllele;
	 cp_SNPlist.Dimension(SNPlist[line].Length());
	 cp_SNPNoAllele.Dimension(SNPNoAllele[line].Length());
	 for(int l=0;l<SNPlist[line].Length();l++)
	 {
	    cp_SNPlist[l] = SNPlist[line][l];
	    cp_SNPNoAllele[l] = SNPNoAllele[line][l];
	 }
	 for(int i=0;i<order.Length();i++)
	 {
	    SNPlist[line][i] = cp_SNPlist[order[i]];
	    //printf("%s\t",SNPlist[line][i].c_str());
	    SNPNoAllele[line][i] = cp_SNPNoAllele[order[i]] ;
	 }
	 //printf("\n");
      }
      line++;
   }
   fclose(samefile);
}

void GroupFromAnnotation::GetGroupFromVCF()
{  
   printf("Parsing annotations from annotated VCF file ...");
   StringArray func;
   func.AddTokens(function,"/");

   FILE * inFile;
   inFile = fopen(vcfInput,"r");
   String buffer;
   StringArray tmp;
   StringIntHash groupHash;
   int geneCount=0;

   while (!feof(inFile))
   {
      buffer.ReadLine(inFile);
      tmp.Clear();
      if(function != "")
      {
	 tmp.AddTokens(buffer, "\t;/= ");
	 for(int i=0;i<tmp.Length();i++)
	 {
	    int result = -1;
	    for(int f =0;f<func.Length();f++)
	    {
	       String tmp_i;
	       tmp_i = tmp[i].SubStr(0,func[f].Length());
	       result = tmp_i.SlowFind(func[f]);
	       if(result==0)
		  break;
	    }
	    if(result == 0)
	    {
	       StringArray sub;
	       sub.AddTokens(tmp[i],":(,");
	       String genename = sub[1];
	       int gene = groupHash.Integer(genename);
	       if(gene==-1)
	       {
		  groupHash.SetInteger(genename,geneCount);
		  geneCount++;
		  annoGroups.Push(genename);
		  chrom.Push(tmp[0]);
	       }
	    }
	 }
      }
      else
      {
	 tmp.AddTokens(buffer, "\t;/ ");
	 for(int i=0;i<tmp.Length();i++)
	 {
	    StringArray tmp_i;
	    tmp_i.AddTokens(tmp[i],"=:(,");
	    if(tmp_i[0]=="ANNO")
	    {
	       String genename = tmp_i[2];
	       int gene = groupHash.Integer(genename);
	       if(gene==-1)
	       {
		  groupHash.SetInteger(genename,geneCount);
		  geneCount++;
		  annoGroups.Push(genename);
		  chrom.Push(tmp[0]);
	       }
	    }
	 }
      }
   }
   fclose(inFile);

   inFile = fopen(vcfInput,"r");
   SNPlist = new StringArray [geneCount];
   SNPNoAllele = new StringArray [geneCount];
   Vector * pos;
   pos = new Vector [geneCount];

   while (!feof(inFile))
   {
      buffer.ReadLine(inFile);
      tmp.Clear();
      if(function != "")
      {
	 tmp.AddTokens(buffer, "\t;/= ");
	 int oldgene = -1;
	 for(int i=0;i<tmp.Length();i++)
	 {
	    int result = -1;
	    for(int f =0;f<func.Length();f++)
	    {
	       String tmp_i;
	       tmp_i = tmp[i].SubStr(0,func[f].Length());
	       result = tmp_i.SlowFind(func[f]);
	       if(result==0)
		  break;
	    }

	    if(result==0)
	    {
	       StringArray sub;
	       sub.AddTokens(tmp[i],":(,");
	       String genename = sub[1];
	       int gene = groupHash.Integer(genename);
	       if(gene==oldgene)
		  continue;
	       oldgene =gene;

	       String markername;
	       if(tmp[0].Find("chr")!=-1)
		  markername = tmp[0].SubStr(3);
	       else
		  markername = tmp[0];

	       markername += ":";
	       markername += tmp[1];
	       markername += ":";
	       markername += tmp[3];
	       markername += ":";
	       markername += tmp[4];
	       SNPlist[gene].Push(markername);
	       SNPNoAllele[gene].Push(tmp[0]+":"+tmp[1]);
	       pos[gene].Push(tmp[1].AsInteger());
	    }
	 }
      }
      else
      {
	 tmp.AddTokens(buffer, "\t;/ ");
	 int oldgene = -1;
	 for(int i=0;i<tmp.Length();i++)
	 {
	    StringArray tmp_i;
	    tmp_i.AddTokens(tmp[i],"=:(,");
	    if(tmp_i[0]=="ANNO")
	    {
	       String genename = tmp_i[2];
	       int gene = groupHash.Integer(genename);
	       if(gene==oldgene)
		  continue;
	       oldgene =gene;

	       String markername;
	       if(tmp[0].Find("chr")!=-1)
		  markername = tmp[0].SubStr(3);
	       else
		  markername = tmp[0];

	       markername += ":";
	       markername += tmp[1];
	       markername += ":";
	       markername += tmp[3];
	       markername += ":";
	       markername += tmp[4];
	       SNPlist[gene].Push(markername);
	       SNPNoAllele[gene].Push(tmp[0]+":"+tmp[1]);
	       pos[gene].Push(tmp[1].AsInteger());
	    }
	 }
      }
   }
   fclose(inFile);

   //sort SNPlist and SNPNoAllele
   for(int line=0;line<geneCount;line++)
   {
      if(SNPlist[line].Length()>1)
      {
	 Vector sorted_pos,order;
	 sorted_pos.Copy(pos[line]);
	 sorted_pos.Sort();
	 order.Dimension(pos[line].Length());
	 for(int i=0;i<sorted_pos.Length();i++)
	 {
	    for(int j=0;j<pos[line].Length();j++)
	    {
	       if(sorted_pos[i]==pos[line][j])
	       {
		  order[i]=j;
		  break;
	       }
	    }
	 }

	 StringArray cp_SNPlist,cp_SNPNoAllele;
	 cp_SNPlist.Dimension(SNPlist[line].Length());
	 cp_SNPNoAllele.Dimension(SNPNoAllele[line].Length());
	 for(int l=0;l<SNPlist[line].Length();l++)
	 {
	    cp_SNPlist[l] = SNPlist[line][l];
	    cp_SNPNoAllele[l] = SNPNoAllele[line][l];
	 }
	 for(int i=0;i<order.Length();i++)
	 {
	    SNPlist[line][i] = cp_SNPlist[order[i]];
	    SNPNoAllele[line][i] = cp_SNPNoAllele[order[i]] ;
	 }
      }
   }
   printf("done!\n");

/* This is for outpupt groupfile from annotated vcf file
   FILE * out;
   String filename = "test.groupfile";
   out = fopen(filename,"wt");
   for(int i=0;i<geneCount;i++)
   {
      fprintf(out,"%s\t",annoGroups[i].c_str());
      for(int m=0;m<SNPlist[i].Length();m++)
         fprintf(out,"%s\t",SNPlist[i][m].c_str());
      fprintf(out,"\n");
   }
   fclose(out);
*/
}

void GroupFromAnnotation::Run()
{ 
   if(vcfInput =="")
      GetGroupFromFile();
   else
      GetGroupFromVCF();
}


